<?php

 
namespace IPS\videobox;

/**
 * Easy video Application Class
 */
class _Application extends \IPS\Application
{
	/**
	 * [Node] Get Icon for tree
	 *
	 * @note	Return the class for the icon (e.g. 'globe')
	 * @return	string|null
	 */
	protected function get__icon()
	{
		return 'play-circle-o';
	}
	
	/**
	 * Default front navigation
	 *
	 * @code
	 	
	 	// Each item...
	 	array(
			'key'		=> 'Example',		// The extension key
			'app'		=> 'core',			// [Optional] The extension application. If ommitted, uses this application	
			'config'	=> array(...),		// [Optional] The configuration for the menu item
			'title'		=> 'SomeLangKey',	// [Optional] If provided, the value of this language key will be copied to menu_item_X
			'children'	=> array(...),		// [Optional] Array of child menu items for this item. Each has the same format.
		)
	 	
	 	return array(
		 	'rootTabs' 		=> array(), // These go in the top row
		 	'browseTabs'	=> array(),	// These go under the Browse tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'browseTabsEnd'	=> array(),	// These go under the Browse tab after all other items on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'activityTabs'	=> array(),	// These go under the Activity tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Activity tab may not exist)
		)
	 * @endcode
	 * @return array
	 */
	public function defaultFrontNavigation()
	{
		return array(
			'rootTabs'		=> array(array( 'key' => 'Videobox' )),
			'browseTabs'	=> array(),
			'browseTabsEnd'	=> array(),
			'activityTabs'	=> array()
		);
	}

	//-----------------------------
	// Creat thumbnail
	//-----------------------------	
	public function createThumb($size, $thumb, $old, $storageExtension)
	{
		try
		{
			\IPS\File::get( '{$storageExtension}', $old )->delete();
			$defaultSize = explode( ':', $size );
			$image = \IPS\Image::create( $thumb->contents() );
			$image->crop(  $defaultSize[0], $defaultSize[1] );	
			$thumbnailName = $thumb->thumbnailName ?: mb_substr( $thumb->originalFilename, 0, mb_strrpos( $thumb->originalFilename, '.' ) ) . '.thumb' . mb_substr( $thumb->originalFilename, mb_strrpos( $thumb->originalFilename, '.' ) );
			return \IPS\File::create( $storageExtension, $thumbnailName, $image, $thumb->thumbnailContainer, FALSE, NULL, $thumb->thumbnailName ? FALSE : TRUE );			
		}
		catch( \Exception $e )
		{
			return null;
		}
	}

	/*-------------------------------------------------------------------------*/
	// Determine which table rows template to use
	/*-------------------------------------------------------------------------*/	
	public function getTableRowsTemplate()
	{
		if( isset( \IPS\Request::i()->cookie['videoboxSize'] ) AND \IPS\Request::i()->cookie['videoboxSize'] == 'list' AND \IPS\Request::i()->controller != 'search' )
		{
			return 'categoryRows_list';
		}
		else
		{
			return 'categoryRows_grid';
		}
	}	
	
	/*-------------------------------------------------------------------------*/
	// Get first image
	/*-------------------------------------------------------------------------*/		
	public function getthumbnail( $content )
	{	
		if (preg_match( '~<img[^>]*(?<!_mce_)src\s?=\s?([\'"])((?:(?!\1).)*)[^>]*>~i', $content, $match ))
		{
			return $match[2];		
		}
		
		preg_match_all( "/(http|https)\:\/\/[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(\/\S*)?/", $content, $url );
		for ( $i = 0 ; $i < count($url[0]) ; $i++ )
		{
			$videoURL = $url[0][$i];
			if (preg_match('#^(?:https?://)?(?:www\.)?(?:m\.)?(?:youtu\.be/|youtube\.com(?:/embed/|/v/|/watch\?v=|/watch\?.+&v=))([\w-]{11})(?:.+)?$#x', $videoURL, $match))
			{
				return "https://img.youtube.com/vi/{$match[1]}/mqdefault.jpg";
			}
			
			if ( preg_match( "/dailymotion.com\/(.*?)/", $videoURL, $match ) )
			{
				$id = strtok(basename($videoURL), '_');
				try
				{
					$request = \IPS\Http\Url::external( "https://api.dailymotion.com/video/" . $id . "?fields=thumbnail_480_url" )->request()->get();
					$output = json_decode($request);
					return $output->thumbnail_480_url;
				}
				catch ( \IPS\Http\Exception $e ) {}
				catch( \IPS\Http\Request\Exception $e ) {}
				catch( \RuntimeException $e ) {}
				return null;
			}
			
			if ( preg_match( "/vimeo.com\/([1-9.-_]+)/", $videoURL, $match ) )
			{
				try
				{
					$request = \IPS\Http\Url::external( "http://vimeo.com/api/v2/video/{$match[1]}.json" )->request()->get();
					$output = json_decode($request);
					$output = $output[0];
					return $output->thumbnail_large;					
				}
				catch ( \IPS\Http\Exception $e ) {}
				catch( \IPS\Http\Request\Exception $e ) {}
				catch( \RuntimeException $e ) {}
				return null;
			}
		}	
		
		return;
	}

	//-----------------------------
	// searchCountry
	//-----------------------------		
	public function searchCountry($code)
	{
		$countrycodes = array ( 'ab' => 'Abkhazian', 'aa' => 'Afar', 'af' => 'Afrikaans', 'sq' => 'Albanian', 'am' => 'Amharic', 'ar' => 'Arabic', 'an' => 'Aragonese', 
								'hy' => 'Armenian', 'as' => 'Assamese', 'ay' => 'Aymara', 'az' => 'Azerbaijani', 'ba' => 'Bashkir', 'eu' => 'Basque', 'bn' => 'Bengali (Bangla)',
								'dz' => 'Bhutani', 'bh' => 'Bihari', 'bi' => 'Bislama', 'br' => 'Breton', 'bg' => 'Bulgarian', 'my' => 'Burmese',
								'be' => 'Byelorussian (Belarusian)', 'km' => 'Cambodian', 'ca' => 'Catalan', 'zh' => 'Chinese (Simplified)', 'zh' => 'Chinese (Traditional)',
								'co' => 'Corsican', 'hr' => 'Croatian', 'cs' => 'Czech', 'da' => 'Danish', 'nl' => 'Dutch', 'en' => 'English', 'eo' => 'Esperanto', 
								'et' => 'Estonian', 'fo' => 'Faeroese', 'fa' => 'Farsi', 'fj' => 'Fiji', 'fi' => 'Finnish', 'fr' => 'French', 'fy' => 'Frisian', 
								'gl' => 'Galician', 'gd' => 'Gaelic (Scottish)', 'gv' => 'Gaelic (Manx)', 'ka' => 'Georgian', 'de' => 'German', 'el' => 'Greek', 
								'kl' => 'Greenlandic', 'gn' => 'Guarani', 'gu' => 'Gujarati', 'ht' => 'Haitian Creole', 'ha' => 'Hausa', 'he' => 'Hebrew', 'hi' => 'Hindi', 
								'hu' => 'Hungarian', 'is' => 'Icelandic', 'io' => 'Ido', 'id' => 'Indonesian', 'ia' => 'Interlingua', 'ie' => 'Interlingue', 
								'iu' => 'Inuktitut', 'ik' => 'Inupiak', 'ga' => 'Irish', 'it' => 'Italian', 'ja' => 'Japanese', 'jv' => 'Javanese', 'kn' => 'Kannada', 
								'ks' => 'Kashmiri', 'kk' => 'Kazakh', 'rw' => 'Kinyarwanda (Ruanda)', 'ky' => 'Kirghiz', 'rn' => 'Kirundi (Rundi)', 'ko' => 'Korean', 
								'ku' => 'Kurdish', 'lo' => 'Laothian', 'la' => 'Latin', 'lv' => 'Latvian (Lettish)', 'li' => 'Limburgish ( Limburger)', 'ln' => 'Lingala', 
								'lt' => 'Lithuanian', 'mk' => 'Macedonian', 'mg' => 'Malagasy', 'ms' => 'Malay', 'ml' => 'Malayalam', 'mt' => 'Maltese', 'mi' => 'Maori', 
								'mr' => 'Marathi', 'mo' => 'Moldavian', 'mn' => 'Mongolian', 'na' => 'Nauru', 'ne' => 'Nepali', 'no' => 'Norwegian', 'oc' => 'Occitan', 
								'or' => 'Oriya', 'om' => 'Oromo (Afan, Galla)', 'ps' => 'Pashto (Pushto)', 'pl' => 'Polish', 'pt' => 'Portuguese', 'pa' => 'Punjabi', 
								'qu' => 'Quechua', 'rm' => 'Rhaeto-Romance', 'ro' => 'Romanian', 'ru' => 'Russian', 'sm' => 'Samoan', 'sg' => 'Sangro', 'sa' => 'Sanskrit', 
								'sr' => 'Serbian', 'sh' => 'Serbo-Croatian', 'st' => 'Sesotho', 'tn' => 'Setswana', 'sn' => 'Shona', 'ii' => 'Sichuan Yi', 'sd' => 'Sindhi', 
								'si' => 'Sinhalese', 'ss' => 'Siswati', 'sk' => 'Slovak', 'sl' => 'Slovenian', 'so' => 'Somali', 'es' => 'Spanish', 'su' => 'Sundanese', 
								'sw' => 'Swahili (Kiswahili)', 'sv' => 'Swedish', 'tl' => 'Tagalog', 'tg' => 'Tajik', 'ta' => 'Tamil', 'tt' => 'Tatar', 'te' => 'Telugu', 
								'th' => 'Thai', 'bo' => 'Tibetan', 'ti' => 'Tigrinya', 'to' => 'Tonga', 'ts' => 'Tsonga', 'tr' => 'Turkish', 'tk' => 'Turkmen', 
								'tw' => 'Twi', 'ug' => 'Uighur', 'uk' => 'Ukrainian', 'ur' => 'Urdu', 'uz' => 'Uzbek', 'vi' => 'Vietnamese', 'vo' => 'Volapük', 
								'wa' => 'Wallon', 'cy' => 'Welsh', 'wo' => 'Wolof', 'xh' => 'Xhosa', 'yi' => 'Yiddish', 'yo' => 'Yoruba', 'zu' => 'Zulu' );
		
		$name = $countrycodes[ mb_strtolower($code) ];
		return $name;
	}	
	
	/*-------------------------------------------------------------------------*/
	// getLinkPicasa - It does not work now because google closed the picasa
	/*-------------------------------------------------------------------------*/
	public function getLinkGoogle($linkgoogle)
	{ 	
		$curArray = array();

		if ( mb_strpos($linkgoogle,'picasaweb.google.com') !== false )
		{
			$curAuthkey = $this->cut_str($linkgoogle,"authkey=","#");	
			$curPhotoID = mb_substr(explode("#",$linkgoogle)[1],0,19);		
			$curUserID = mb_substr(explode("https://picasaweb.google.com/",$linkgoogle)[1],0,21);		
			$curRSSURL ="https://picasaweb.google.com/data/entry/api/user/".$curUserID."/photoid/".$curPhotoID."?authkey=". $curAuthkey. "&time=". rand(5000,100000);	
			$curTemp = @file_get_contents($curRSSURL);

			if ($curTemp <> "") 
			{			
				$curList = explode('media:content',$curTemp);
				if (!$curList) return "broken";
				foreach ($curList as $curl) 
				{
					$curl = trim($this->cut_str($curl,"'" , "'"));
					$curl = urldecode($curl);
					$curl = str_replace("&amp;","&",$curl);
					if ($curl <> "")
					{				
						if (mb_strpos($curl,'itag=37') !== false) {$cur1080p=$curl;}
						if (mb_strpos($curl,'itag=22') !== false) {$cur720p=$curl;}
						if (mb_strpos($curl,'itag=18') !== false) {$cur360p=$curl;}
						if (mb_strpos($curl,'m37') !== false) {$cur1080p=$curl;}
						if (mb_strpos($curl,'m22') !== false) {$cur720p=$curl;}
						if (mb_strpos($curl,'m18') !== false) {$cur360p=$curl;}
					}
				}

				if ($cur1080p) array_push($curArray,array("link"=>"$cur1080p", "res"=>"1080", "type"=> "video/mp4"));
				if ($cur720p) array_push($curArray,array("link"=>"$cur720p", "res"=>"720", "type"=> "video/mp4"));
				if ($cur360p) array_push($curArray,array("link"=>"$cur360p", "res"=>"360", "type"=> "video/mp4"));
			}	
		}
		else  
		{
			$get = $this->_curl($linkgoogle);
			$cut_source = $this->cut_str($get,'"fmt_stream_map"','url_encoded_fmt_stream_map');
			$cut_linkarray = explode ('|',$cut_source);

			foreach ($cut_linkarray as $curl)
			{
				$curl = urldecode($curl);
				$curl = str_replace('\u003d', '=', $curl);
				$curl = str_replace('\u0026', '&',$curl);			
				if ($curl <> "" )
				{
					if (mb_strpos($curl,'itag=37') !== false) { $cur1080p="https://redirector.googlevideo.com/videoplayback".$this->cut_str($curl,'videoplayback','&pl');}
					if (mb_strpos($curl,'itag=22') !== false)  { $cur720p="https://redirector.googlevideo.com/videoplayback".$this->cut_str($curl,'videoplayback','&pl');}
					if (mb_strpos($curl,'itag=59') !== false) { $cur480p="https://redirector.googlevideo.com/videoplayback".$this->cut_str($curl,'videoplayback','&pl');}
					if (mb_strpos($curl,'itag=18') !== false)  { $cur360p="https://redirector.googlevideo.com/videoplayback".$this->cut_str($curl,'videoplayback','&pl');}
				}	
			}
			if ($cur1080p) array_push($curArray,array("link"=>"$cur1080p", "res"=>"1080","type"=>"video/mp4"));
			if ($cur720p) array_push($curArray,array("link"=>"$cur720p", "res"=>"720","type"=>"video/mp4"));
			if ($cur480p) array_push($curArray,array("link"=>"$cur480p", "res"=>"480","type"=>"video/x-flv"));
			if ($cur360p) array_push($curArray,array("link"=>"$cur360p", "res"=>"360","type"=>"video/mp4"));
		}
		
		return $curArray;
	}
	
	protected function cut_str($str, $left, $right)
	{
		$str = mb_substr(mb_stristr($str, $left) , mb_strlen($left));
		$leftLen = mb_strlen(mb_stristr($str, $right));
		$leftLen = $leftLen ? -($leftLen) : mb_strlen($str);
		$str = mb_substr($str, 0, $leftLen);
		return $str;
	}	
	
	protected function _curl($url)
	{
		$ch = @curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		$head[] = "Connection: keep-alive";
		$head[] = "Keep-Alive: 300";
		$head[] = "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7";
		$head[] = "Accept-Language: en-us,en;q=0.5";
		curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062.124 Safari/537.36');
		curl_setopt($ch, CURLOPT_HTTPHEADER, $head);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 60);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
		$page = curl_exec($ch);
		curl_close($ch);
		return $page;
	}
	
	//-----------------------------
	// Replace Media Tag
	//-----------------------------
	public function customPlayer($url)
	{
		$url = mb_substr($url, -4) == '|end' ? mb_substr($url, 0, -4) : $url;
		
		foreach( \IPS\Db::i()->select( '*', 'videobox_mediatags', NULL ) as $row )
		{	
			$match = $row['vbmt_match'];
			// $match = str_replace( '?', '\?', $match );		
			$match = preg_replace( "#{[0-9]}#", "(.*?)", $match );
			
			$data[$row['vbmt_name']] = array( 'match' => $match, 'replace' => $row['vbmt_replace'] );
		}

		if ( is_array( $data ) AND count( $data ) )
		{
			foreach( $data as $k => $r )
			{
				if( preg_match( "#^" . $r['match'] . "$#is", $url ) )
				{				
					return preg_replace( "#^" . $r['match'] . "$#is", "<div class='ipsEmbeddedVideo' contenteditable='false'>" . $r['replace'] . "</div>", $url );
				}
			}
		}
		return;
	}
	
	//-----------------------------
	// Live stream info
	//-----------------------------	
	public function lstreamInfo($video, $save = false)
	{
		if ( $video->livestream )
		{
			$name = trim( $video->livestream );
			if( mb_substr( $video->livestream, -1 ) == '/' )
			{
				$name = mb_substr( $video->livestream, 0, -1 );
			}
			$name = explode( '/', $name );
			$name = end( $name );
			$data = $this->requestStreamData($video->livestream, $name);
		}

		if ( $data && $save == true )
		{
			$video->livestream_online = $data['online'];
			$video->livestream_data = @json_encode( $data );
			$video->save();
		}
		
		return $data;
	}
	
	public function requestStreamData($url, $name, $returnRequest=0)
	{
		$data = array();
		
		if ( mb_strpos( $url, 'twitch.tv' ) !== false )
		{
			$request = $this->get_stream_data("https://api.twitch.tv/kraken/streams?channel=" . $name . "&stream_type=all&limit=100&client_id=" . \IPS\Settings::i()->vb_conf_lv_twitch_api);
			$data = $this->twitchData($request['streams'][0], $name);
		}
		elseif ( mb_strpos( $url, 'smashcast.tv' ) !== false )
		{
			$request = $this->get_stream_data("https://api.smashcast.tv/media/live/" . $name . ".json");
			$data = $this->smashcastData($request['livestream'][0], $name);
		}			
		elseif ( mb_strpos( $url, 'mixer.com' ) !== false )
		{
			$request = $this->get_stream_data("https://mixer.com/api/v1/channels?where=token:in:" . $name . "&limit=50");	
			$data = $this->mixerData($request[0], $name);		
		}

		return $returnRequest == 1 ? $request : $data;
	}
	
	public function twitchData($sdata, $name)
	{
		$offlineThumb = "//static-cdn.jtvnw.net/ttv-static/404_preview-320x180.jpg";
		
		$data = array( 	'online' 	=> !$sdata ? 'offline' : 'online',
						'viewers'	=> $sdata ? $sdata['viewers'] : null,
						'views'		=> $sdata ? $sdata['channel']['views'] : null,
						'followers'	=> $sdata ? $sdata['channel']['followers'] : null,
						'status'	=> $sdata ? $sdata['channel']['status'] : null,
						'image'		=> $sdata['preview']['medium'] ? $sdata['preview']['medium'] : $offlineThumb,
						'thumbnail'	=> str_replace("{width}x{height}", "320x180", $sdata ? $sdata['preview']['template'] : $offlineThumb),
						'banner'	=> str_replace("{width}x{height}", str_replace( ":", "x", \IPS\Settings::i()->vb_conf_featured_size ), $sdata ? $sdata['preview']['template'] : $offlineThumb),
						'media'		=> "//www.twitch.tv/". $name . "/embed",
						'chat'		=> "//www.twitch.tv/". $name . "/chat?popout=",		
		);
		
		return $data;
	}
	
	public function smashcastData( $sdata, $name )
	{
		$offlineThumb = (string) \IPS\Theme::i()->resource( 'livestream.png', 'videobox', 'front' );
		
		$data = array();
		$status = $sdata['media_is_live'];
		$data = array( 	'online' 	=> $status == 1 ? 'online' : 'offline',
						'viewers'	=> $sdata['media_views'],
						'views'		=> '',
						'followers'	=> $sdata['channel']['followers'],
						'status'	=> $sdata['media_status'],
						'image'		=> $status == 1 ? "//edge.sf.hitbox.tv" . $sdata['media_thumbnail'] : $offlineThumb,
						'thumbnail'	=> $status == 1 ? "//edge.sf.hitbox.tv" . $sdata['media_thumbnail'] : $offlineThumb,
						'banner'	=> $status == 1 ? "//edge.sf.hitbox.tv" . $sdata['media_thumbnail_large'] : $offlineThumb,
						'media'		=> "//www.smashcast.tv/embed/" . $name,
						'chat'		=> "//www.smashcast.tv/embedchat/" . $name,
		);	
		return $data;
	}
	
	public function mixerData($sdata, $name)
	{
		$offlineThumb = (string) \IPS\Theme::i()->resource( 'livestream.png', 'videobox', 'front' );

		$data = array( 	'online' 	=> $sdata['online'] ? 'online' : 'offline',
						'viewers'	=> $sdata['online'] ? $sdata['viewersCurrent'] : null,
						'views'		=> $sdata ? $sdata['viewersTotal'] : null,
						'followers'	=> $sdata ? $sdata['numFollowers'] : null,
						'status'	=> null,
						'image'		=> $sdata['online'] ? "https://thumbs.mixer.com/channel/" . $sdata['id'] . ".small.jpg" : $offlineThumb,
						'thumbnail'	=> $sdata['online'] ? "https://thumbs.mixer.com/channel/" . $sdata['id'] . ".small.jpg" : $offlineThumb,
						'banner'	=> $sdata['online'] ? "https://thumbs.mixer.com/channel/" . $sdata['id'] . ".big.jpg" : $offlineThumb,
						'media'		=> "//mixer.com/embed/player/". $name,
						'chat'		=> "//mixer.com/embed/chat/". $name,		
		);

		return $data;
	}
	
	private function get_stream_data($url) 
	{
		$ch = \curl_init();
		$timeout = 5;
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);	
		$data = curl_exec($ch);
		curl_close($ch);
		
		return json_decode($data, true);
	}	
}