<?php


namespace IPS\videobox\modules\front\video;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{	
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{	
		/* What's new */
		if ( \IPS\Settings::i()->vb_conf_index_new_on == 1 )
		{
			$where = array();
			if ( \IPS\Settings::i()->vb_conf_index_new_cat && \IPS\Settings::i()->vb_conf_index_new_cat != 0 )
			{
				$where[] = array( 'v_cat IN (' . \IPS\Settings::i()->vb_conf_index_new_cat . ')' );
			}			
			$url = \IPS\Http\Url::internal( 'app=videobox', 'front', 'videobox' );
			$table = new \IPS\videobox\Video\Table( 'IPS\videobox\Video', $url, $where );
			$table->limit = \IPS\Settings::i()->vb_conf_index_new_items > 0 ? \IPS\Settings::i()->vb_conf_index_new_items : 30;
			$table->title = \IPS\Member::loggedIn()->language()->addToStack('videobox_whats_new');
			$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'videoTable' );
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), \IPS\Application::load('videobox')->getTableRowsTemplate() );
			$table->sortOptions = array_merge( $table->sortOptions, array( 'online' => 'v_livestream_online' ) );		
			$table->sortBy = $table->sortBy ?: 'v_updated';
			$table->sortDirection = $table->sortBy == 'videobox_videos.v_title' ? 'asc' : 'desc';
			$new = (string) $table;
		}
		
		/* Add RSS feed */
		if ( \IPS\Settings::i()->vb_conf_rss )
		{
			\IPS\Output::i()->rssFeeds['vb_rss_title'] = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index&do=rss', 'front', 'videobox_rss' );
		}
		
		/* Online User Location */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=videobox', 'front', 'videobox' ), array(), 'loc_videobox_browsing' );

		/* Display */
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('__app_videobox');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->index($new);
	}
	
	/**
	 * Latest Videos RSS
	 *
	 * @return	void
	 */
	protected function rss()
	{
		if( !\IPS\Settings::i()->vb_conf_rss )
		{
			\IPS\Output::i()->error( 'rss_offline', '2BIMVB105/1', 404, 'rss_offline_admin' );
		}

		$document = \IPS\Xml\Rss::newDocument( \IPS\Http\Url::internal( 'app=videobox&module=video&controller=index', 'front', 'videobox' ), \IPS\Member::loggedIn()->language()->get('vb_rss_title'), \IPS\Member::loggedIn()->language()->get('vb_rss_title') );
		
		foreach ( \IPS\videobox\Video::getItemsWithPermission() as $video )
		{
			$thumbnail = "<img src=\"{$video->thumbnail('medium')}\" style='align: center; margin: 0 auto;'><br>";
			$document->addItem( $video->title, $video->url(), $thumbnail . $video->content, \IPS\DateTime::ts( $video->updated ), $video->id );
		}
		
		/* @note application/rss+xml is not a registered IANA mime-type so we need to stick with text/xml for RSS */
		\IPS\Output::i()->sendOutput( $document->asXML(), 200, 'text/xml' );
	}	
}