<?php
/**
 * @brief		FeaturedVideos Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		04 Apr 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * FeaturedVideos Widget
 */
class _FeaturedVideos extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'FeaturedVideos';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
		{
	 		$form = new \IPS\Helpers\Form;
 		}
		
		// Custom Fields
		$customFields = array();
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}	
		
		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_featured_fields', isset( $this->configuration['vb_conf_featured_fields'] ) ? $this->configuration['vb_conf_featured_fields'] : 0, FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
		}	
		
		// Items
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_featured_items', $this->configuration['vb_conf_featured_items'] ? $this->configuration['vb_conf_featured_items'] : 10, FALSE, array() ) );
		
		// Container
		$form->add( new \IPS\Helpers\Form\Node( 'vb_conf_featured_cat_to_show', isset( $this->configuration['vb_conf_featured_cat_to_show'] ) ? $this->configuration['vb_conf_featured_cat_to_show'] : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true,
			'clubs'			  => true,
		) ) );	
		
		// Status
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vb_conf_featured_status', isset( $this->configuration['vb_conf_featured_status'] ) ? $this->configuration['vb_conf_featured_status'] : array( 'featured' ), FALSE, array(
			'options' => array(
			    'closed'      => 'vb_conf_featured_status_closed',
			    'pinned'      => 'vb_conf_featured_status_pinned',
			    'hidden'      => 'vb_conf_featured_status_hidden',
			    'featured'    => 'vb_conf_featured_status_featured',
			)
		) ) );		
		
		// Sorting
		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_featured_sort_on', isset( $this->configuration['vb_conf_featured_sort_on'] ) ? $this->configuration['vb_conf_featured_sort_on'] : 'v_started', FALSE, array(
            'options' => array(
	            'v_started'     		=> 'vb_conf_featured_sort_on_started',
				'v_updated'				=> 'vb_conf_featured_sort_on_updated',
	            'v_title'    			=> 'vb_conf_featured_sort_on_title',
	            'v_rating_average'		=> 'vb_conf_featured_sort_on_rating',
	            'v_comments'  			=> 'vb_conf_featured_sort_on_comments',
				'v_last_comment'		=> 'vb_conf_featured_sort_on_last_comment',
	            'v_views'       		=> 'vb_conf_featured_sort_on_views',
            )
        ) ), NULL, NULL, NULL, 'vb_conf_featured_sort_on' );

		$form->add( new \IPS\Helpers\Form\Select( 'vb_conf_featured_sort_dir', isset( $this->configuration['vb_conf_featured_sort_dir'] ) ? $this->configuration['vb_conf_featured_sort_dir'] : 'desc', FALSE, array(
            'options' => array(
	            'desc'   => 'vb_conf_featured_sort_dir_dsc',
	            'asc'    => 'vb_conf_featured_sort_dir_asc'
            )
        ) ) );
		
		// Output form
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( is_array( $values['vb_conf_featured_cat_to_show'] ) )
 		{
	 		$values['vb_conf_featured_cat_to_show'] = array_keys( $values['vb_conf_featured_cat_to_show'] );
 		}
 				
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		$where = array();
		$order = NULL;		
		$limit = isset( $this->configuration['vb_conf_featured_items'] ) ? $this->configuration['vb_conf_featured_items'] : 9;
		
		if ( !empty( $this->configuration['vb_conf_featured_cat_to_show'] ) )
		{
			$where[] = array( 'v_cat IN (' . implode( ",", $this->configuration['vb_conf_featured_cat_to_show'] ) . ')' );
		}
		
		if ( !empty( $this->configuration['vb_conf_featured_status'] ) )
		{
			$status = $this->configuration['vb_conf_featured_status'];

			if ( in_array( 'closed', $status ) )
			{
				$where[] = array( "v_open=0" );
			}

			if ( in_array( 'hidden', $status ) )
			{
				$includeHidden = true;
			}

			if ( in_array( 'featured', $status ) )
			{
				$where[] = array( 'v_featured=1' );
			}

			if ( in_array( 'pinned', $status ) )
			{
				$where[] = array( 'v_pinned=1' );
			}
		}

		if ( isset( $this->configuration['vb_conf_featured_sort_on'] ) and isset( $this->configuration['vb_conf_featured_sort_dir'] ) )
		{
			$order = $this->configuration['vb_conf_featured_sort_on'] . ' ' . $this->configuration['vb_conf_featured_sort_dir'];
		}		
		
		$videos = \IPS\videobox\Video::getItemsWithPermission( $where, $order, $limit );
		
		if ( count( $videos ) )
		{
			return $this->output( $this->configuration, $videos );
		}
		else
		{
			return '';
		}			
	}
}