<?php
/**
 * @brief		HighestRated Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		05 Jan 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * HighestRated Widget
 */
class _HighestRated extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'HighestRated';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
 		{
	 		$form = new \IPS\Helpers\Form;
 		} 
 		
		$form->add( new \IPS\Helpers\Form\Number( 'number_to_show', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5, TRUE ) );
		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		$categories = array();

		foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', array( 'app=? and perm_type=? and (' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\videobox\Category::$permissionMap['read'], \IPS\Member::loggedIn()->groups ) . ' OR ' . 'perm_' . \IPS\videobox\Category::$permissionMap['read'] . '=? )', 'videobox', 'category', '*' ) ) as $category )
		{
			$categories[]	= $category;
		}

		if( !count( $categories ) )
		{
			return '';
		}

		foreach ( array( 'week' => 'P1W', 'month' => 'P1M', 'year' => 'P1Y', 'all' => NULL ) as $time => $interval )
		{
			$where = array( array( 'v_cat IN(' . implode( ',', $categories ) . ')' ) );
			if ( $interval )
			{
				$where[] = array( 'v_started>? AND v_rating_average>?', \IPS\DateTime::create()->sub( new \DateInterval( $interval ) )->getTimestamp(), 0 );
			}
			
			$ids = array();

			foreach( \IPS\Db::i()->select( '*', 'videobox_videos', $where, 'v_rating_average DESC', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 5 ) as $vd )
			{
				$ids[] = $vd['v_id'];
			}

			if( count( $ids ) )
			{
				$$time = new \IPS\Patterns\ActiveRecordIterator(
					\IPS\Db::i()->select(
						'*',
						'videobox_videos',
						'v_id IN(' . implode( ',', $ids ) . ')'
					),
					'IPS\videobox\Video'
				);
			}
			else
			{
				$$time = array();
			}			
		}
	
		return $this->output( $week, $month, $year, $all );
	}
}