<?php
/**
 * @brief		RandomVideos Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		04 Apr 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * RandomVideos Widget
 */
class _RandomVideos extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'RandomVideos';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
 		{
	 		$form = new \IPS\Helpers\Form;
 		} 		
		$form->add( new \IPS\Helpers\Form\Number( 'vb_conf_random_items', $this->configuration['vb_conf_random_items'] ? $this->configuration['vb_conf_random_items'] : 30, FALSE, array() ) );			
		$form->add( new \IPS\Helpers\Form\Node( 'vb_conf_random_cat', isset( $this->configuration['vb_conf_random_cat'] ) ? $this->configuration['vb_conf_random_cat'] : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true,
			'clubs'			  => true,
		) ) );			
		return $form;		
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( is_array( $values['vb_conf_random_cat'] ) )
 		{
	 		$values['vb_conf_random_cat'] = array_keys( $values['vb_conf_random_cat'] );
 		}
 		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		$where = array();
		
		if ( !empty( $this->configuration['vb_conf_random_cat'] ) )
		{
			$where[] = array( 'v_cat IN (' . implode( ",", $this->configuration['vb_conf_random_cat'] ) . ')' );
		}	

		$random = \IPS\videobox\Video::getItemsWithPermission( $where, 'RAND()', $this->configuration['vb_conf_random_items'] ? $this->configuration['vb_conf_random_items'] : 30 );
		
		if ( count( $random ) )
		{
			return $this->output( $random );
		}
		else
		{
			return '';
		}
	}
}