<?php
/**
 * @brief		Videos Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	videobox
 * @since		10 Mar 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\videobox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Videos Widget
 */
class _Videos extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'Videos';
	
	/**
	 * @brief	App
	 */
	public $app = 'videobox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
		// Use this to perform any set up and to assign a template that is not in the following format:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
 		{
	 		$form = new \IPS\Helpers\Form;
 		} 
		
 		$form->add( new \IPS\Helpers\Form\Text( 'vbox_block_title', $this->configuration['vbox_block_title'], FALSE ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_block_cansee', $this->configuration['vbox_block_cansee'] == 'all' ? 'all' : $this->configuration['vbox_block_cansee'], FALSE, array( 
			'options' 		=> \IPS\Member\Group::groups(), 
			'parse' 		=> 'normal', 
			'multiple' 		=> true, 
			'unlimited' 	=> 'all', 
			'unlimitedLang' => 'all_groups' 
		) ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'number_to_show', isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 9, TRUE ) );
		
		$form->add( new \IPS\Helpers\Form\Node( 'vbox_cat_to_show', isset( $this->configuration['vbox_cat_to_show'] ) ? $this->configuration['vbox_cat_to_show'] : 0, FALSE, array(
			'class'           => '\IPS\videobox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true,
			'clubs'			  => true,
		) ) );	
		
		$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vbox_lstream_status', isset( $this->configuration['vbox_lstream_status'] ) ? $this->configuration['vbox_lstream_status'] : array(), FALSE, array(
			'options' => array(
				'online'       => 'vbox_lstream_online',
			    'offline'      => 'vbox_lstream_offline'
			)
		) ) );		
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vbox_field_on', $this->configuration['vbox_field_on'], FALSE, array( 'togglesOn' => array( 'vbox_field', 'vbox_field_cond', 'vbox_field_val' ) ) ) );
		
		$customFields = array();
		
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}	
		
		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'vbox_field', $this->configuration['vbox_field'], FALSE, array( 'options' => $customFields, 'multiple' => FALSE), NULL, NULL, NULL, 'vbox_field' ) );
		}
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_field_cond', $this->configuration['vbox_field_cond'] ? $this->configuration['vbox_field_cond'] : 'equal', TRUE, array(
            'options' => array(
	            'like'   	=> 'v_qs_mode_like',			
	            'equal'   	=> 'v_qs_mode_equal',
	            'greater'   => 'v_qs_mode_greater',
	            'lessthan'  => 'v_qs_mode_lessthan',
	            'different'	=> 'v_qs_mode_different',					
            )
        ), NULL, NULL, NULL, 'vbox_field_cond' ) );
		
		$form->add( new \IPS\Helpers\Form\Text( 'vbox_field_val', $this->configuration['vbox_field_val'], FALSE, array(), NULL, NULL, NULL, 'vbox_field_cond' ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_sort_on', isset( $this->configuration['vbox_sort_on'] ) ? $this->configuration['vbox_sort_on'] : 'v_started', FALSE, array(
            'options' => array(
	            'v_started'     		=> 'vbox_sort_on_started',
				'v_updated'				=> 'vbox_sort_on_updated',
	            'v_title'    			=> 'vbox_sort_on_title',
	            'v_rating_average'		=> 'vbox_sort_on_rating',
	            'v_comments'  			=> 'vbox_sort_on_comments',
				'v_last_comment'		=> 'vbox_sort_on_last_comment',
	            'v_views'       		=> 'vbox_sort_on_views',
				'v_livestream_online'	=> 'vbox_lstream_status',
            )
        ) ), NULL, NULL, NULL, 'vbox_sort_on' );

		$form->add( new \IPS\Helpers\Form\Select( 'vbox_sort_dir', isset( $this->configuration['vbox_sort_dir'] ) ? $this->configuration['vbox_sort_dir'] : 'desc', FALSE, array(
            'options' => array(
	            'desc'   => 'vbox_sort_dir_dsc',
	            'asc'    => 'vbox_sort_dir_asc'
            )
        ) ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vbox_block_theme', ( $this->configuration['vbox_block_theme'] and $this->configuration['vbox_block_theme'] != '*' ) ? $this->configuration['vbox_block_theme'] : 'default', FALSE, array(
            'options' => array(
				'default'		=> 'vbox_block_theme_default',			
	            'bigthumb'    	=> 'vbox_block_theme_bigthumb'
            )
        ) ) );
		
		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( is_array( $values['vbox_cat_to_show'] ) )
 		{
	 		$values['vbox_cat_to_show'] = array_keys( $values['vbox_cat_to_show'] );
 		}
 		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( $this->configuration['vbox_block_cansee'] != 'all' AND isset( $this->configuration['vbox_block_cansee'] ) and is_array( $this->configuration['vbox_block_cansee'] ) )
		{
			if( ! \IPS\Member::loggedIn()->inGroup( $this->configuration['vbox_block_cansee'] ) )
			{
				return '';
			}
		}
		
		$where = array();
		$order = NULL;		
		$limit = isset( $this->configuration['number_to_show'] ) ? $this->configuration['number_to_show'] : 9;
		
		if ( !empty( $this->configuration['vbox_cat_to_show'] ) )
		{
			$where[] = array( 'v_cat IN (' . implode( ",", $this->configuration['vbox_cat_to_show'] ) . ')' );
		}
		
		if ( !empty( $this->configuration['vbox_lstream_status'] ) )
		{
			$status = $this->configuration['vbox_lstream_status'];

			if ( in_array( 'online', $status ) and ! in_array( 'offline', $status ) )
			{
				$where[] = array( "v_livestream_online='online'" );
			}
			
			if ( in_array( 'offline', $status ) and ! in_array( 'online', $status ) )
			{
				$where[] = array( "v_livestream_online='offline'" );
			}		
		}

		if ( $this->configuration['vbox_field_on'] == 1 )
		{
			try
			{
				$field = \IPS\videobox\Field::load( $this->configuration['vbox_field'] );
			}
			catch ( \OutOfRangeException $e ) {}

			if ( $field )
			{		
				# Search term
				$searchterm = mb_strtolower( $this->configuration['vbox_field_val'] );

				if ( $field->type == 'CheckboxSet' )
				{
					foreach( json_decode($field->content) as $k => $val )
					{
						if ( mb_strtolower($val) == $searchterm )
						{
							$query = "field_{$field->id} LIKE '" . $k . ",%' OR field_{$field->id} LIKE '%," . $k . "' OR field_{$field->id} LIKE '%," . $k . ",%'";
							break;
						}
					}
				}
				else
				{
					switch ( $this->configuration['vbox_field_cond'] )
					{
						case 'like':
							$query = "field_{$field->id} LIKE '%" . $searchterm . "%'";
							break;				
						case 'equal':
							$query = "field_{$field->id}='" . $searchterm . "'";
							break;
						case 'greater':
							$query = "field_{$field->id}>='" . $searchterm . "'";
							break;
						case 'lessthan':
							$query = "field_{$field->id}<='" . $searchterm . "'";					
							break;
						case 'different':
							$query = "field_{$field->id}!='" . $searchterm . "'";					
							break;			
					}	
				}
				foreach ( \IPS\Db::i()->select( "lower(field_{$field->id}), video_id", "videobox_ccontent", $query, $order, array(0, $limit) ) as $v )
				{
					$tid[] = $v['video_id'];
				}
				
				$where[] = count( $tid ) > 0 ? array( "v_id IN (" . implode( ',', $tid ) . ")" ) : array( "v_id=0" );
			}
		}

		if ( isset( $this->configuration['vbox_sort_on'] ) and isset( $this->configuration['vbox_sort_dir'] ) )
		{
			$order = $this->configuration['vbox_sort_on'] . ' ' . $this->configuration['vbox_sort_dir'];
		}		
		
		$videos = \IPS\videobox\Video::getItemsWithPermission( $where, $order, $limit );
		if ( count( $videos ) )
		{
			return $this->output( $this->configuration, $videos );
		}
		else
		{
			return '';
		}		
	}
}